@@ -23,6 +23,7 @@
  */
 
 #define _GNU_SOURCE 1
+#include "sysdeps.h"
 #include "va_glx_private.h"
 #include "va_glx_impl.h"
 #include <stdio.h>
@@ -190,18 +191,40 @@ static int check_extension(const char *name, const char *ext)
     return 0;
 }
 
+static int check_extension3(const char *name)
+{
+    int nbExtensions, i;
+    PFNGLGETSTRINGIPROC glGetStringi = 0;
+
+    glGetStringi = (PFNGLGETSTRINGIPROC) get_proc_address("glGetStringi");
+    if(!glGetStringi)
+        return 0;
+
+
+    glGetIntegerv(GL_NUM_EXTENSIONS, &nbExtensions);
+    for(i = 0; i < nbExtensions; i++)
+    {
+        const GLubyte *strExtension = glGetStringi(GL_EXTENSIONS, i);
+        if(strcmp((const char *) strExtension, name) == 0)
+            return 1;
+    }
+
+    return 0;
+}
+
 static int check_tfp_extensions(VADriverContextP ctx)
 {
     const char *gl_extensions;
     const char *glx_extensions;
 
     gl_extensions = (const char *)glGetString(GL_EXTENSIONS);
-    if (!check_extension("GL_ARB_texture_non_power_of_two", gl_extensions))
+    if (!check_extension("GL_ARB_texture_non_power_of_two", gl_extensions) && !check_extension3("GL_ARB_texture_non_power_of_two"))
         return 0;
 
-    glx_extensions = glXQueryExtensionsString((Display *)ctx->native_dpy, ctx->x11_screen);
+    glx_extensions = glXQueryExtensionsString(ctx->native_dpy, ctx->x11_screen);
     if (!check_extension("GLX_EXT_texture_from_pixmap", glx_extensions))
         return 0;
+
     return 1;
 }
 
@@ -210,10 +233,11 @@ static int check_fbo_extensions(VADriverContextP ctx)
     const char *gl_extensions;
 
     gl_extensions = (const char *)glGetString(GL_EXTENSIONS);
-    if (check_extension("GL_ARB_framebuffer_object", gl_extensions))
+    if (check_extension("GL_ARB_framebuffer_object", gl_extensions) || check_extension3("GL_ARB_framebuffer_object"))
         return 1;
-    if (check_extension("GL_EXT_framebuffer_object", gl_extensions))
+    if (check_extension("GL_EXT_framebuffer_object", gl_extensions) || check_extension3("GL_EXT_framebuffer_object"))
         return 1;
+
     return 0;
 }
 
@@ -340,8 +364,14 @@ gl_create_context(VADriverContextP ctx, OpenGLContextStateP parent)
     if (!cs)
         goto error;
 
-    cs->display = (Display *)ctx->native_dpy;
-    cs->window  = parent ? parent->window : None;
+    if (parent) {
+        cs->display = parent->display;
+        cs->window  = parent->window;
+    }
+    else {
+        cs->display = ctx->native_dpy;
+        cs->window  = None;
+    }
     cs->context = NULL;
 
     if (parent && parent->context) {
@@ -357,8 +387,8 @@ gl_create_context(VADriverContextP ctx, OpenGLContextStateP parent)
             goto choose_fbconfig;
 
         fbconfigs = glXGetFBConfigs(
-            (Display *)ctx->native_dpy,
-            ctx->x11_screen,
+            parent->display,
+            DefaultScreen(parent->display),
             &n_fbconfigs
         );
         if (!fbconfigs)
@@ -367,7 +397,7 @@ gl_create_context(VADriverContextP ctx, OpenGLContextStateP parent)
         /* Find out a GLXFBConfig compatible with the parent context */
         for (n = 0; n < n_fbconfigs; n++) {
             status = glXGetFBConfigAttrib(
-                (Display *)ctx->native_dpy,
+                cs->display,
                 fbconfigs[n],
                 GLX_FBCONFIG_ID, &val
             );
@@ -380,7 +410,7 @@ gl_create_context(VADriverContextP ctx, OpenGLContextStateP parent)
     else {
     choose_fbconfig:
         fbconfigs = glXChooseFBConfig(
-            (Display *)ctx->native_dpy,
+            ctx->native_dpy,
             ctx->x11_screen,
             fbconfig_attrs, &n_fbconfigs
         );
@@ -392,7 +422,7 @@ gl_create_context(VADriverContextP ctx, OpenGLContextStateP parent)
     }
 
     cs->context = glXCreateNewContext(
-        (Display *)ctx->native_dpy,
+        cs->display,
         fbconfigs[n],
         GLX_RGBA_TYPE,
         parent ? parent->context : NULL,
@@ -470,12 +500,12 @@ static int create_tfp_surface(VADriverContextP ctx, VASurfaceGLXP pSurfaceGLX)
     int                  *attrib;
     int                   n_fbconfig_attrs;
 
-    root_window = RootWindow((Display *)ctx->native_dpy, ctx->x11_screen);
-    XGetWindowAttributes((Display *)ctx->native_dpy, root_window, &wattr);
+    root_window = RootWindow(ctx->native_dpy, ctx->x11_screen);
+    XGetWindowAttributes(ctx->native_dpy, root_window, &wattr);
     if (wattr.depth != 24 && wattr.depth != 32)
         return 0;
     pixmap = XCreatePixmap(
-        (Display *)ctx->native_dpy,
+        ctx->native_dpy,
         root_window,
         width,
         height,
@@ -494,11 +524,19 @@ static int create_tfp_surface(VADriverContextP ctx, VASurfaceGLXP pSurfaceGLX)
         GLX_RED_SIZE,           8,
         GLX_GREEN_SIZE,         8,
         GLX_BLUE_SIZE,          8,
+        /*
+         * depth test isn't enabled in the implementaion of VA GLX,
+         * so depth buffer is unnecessary. However to workaround a
+         * bug in older verson of xorg-server, always require a depth
+         * buffer.
+         *
+         * See https://bugs.freedesktop.org/show_bug.cgi?id=76755
+         */
+        GLX_DEPTH_SIZE,         1,
         GL_NONE,
     };
     for (attrib = fbconfig_attrs; *attrib != GL_NONE; attrib += 2)
         ;
-    *attrib++ = GLX_DEPTH_SIZE;                 *attrib++ = wattr.depth;
     if (wattr.depth == 32) {
     *attrib++ = GLX_ALPHA_SIZE;                 *attrib++ = 8;
     *attrib++ = GLX_BIND_TO_TEXTURE_RGBA_EXT;   *attrib++ = GL_TRUE;
@@ -509,7 +547,7 @@ static int create_tfp_surface(VADriverContextP ctx, VASurfaceGLXP pSurfaceGLX)
     *attrib++ = GL_NONE;
 
     fbconfig = glXChooseFBConfig(
-        (Display *)ctx->native_dpy,
+        ctx->native_dpy,
         ctx->x11_screen,
         fbconfig_attrs,
         &n_fbconfig_attrs
@@ -533,7 +571,7 @@ static int create_tfp_surface(VADriverContextP ctx, VASurfaceGLXP pSurfaceGLX)
 
     x11_trap_errors();
     glx_pixmap = pOpenGLVTable->glx_create_pixmap(
-        (Display *)ctx->native_dpy,
+        ctx->native_dpy,
         fbconfig[0],
         pixmap,
         pixmap_attrs
@@ -561,12 +599,12 @@ static void destroy_tfp_surface(VADriverContextP ctx, VASurfaceGLXP pSurfaceGLX)
     }
 
     if (pSurfaceGLX->glx_pixmap) {
-        pOpenGLVTable->glx_destroy_pixmap((Display *)ctx->native_dpy, pSurfaceGLX->glx_pixmap);
+        pOpenGLVTable->glx_destroy_pixmap(ctx->native_dpy, pSurfaceGLX->glx_pixmap);
         pSurfaceGLX->glx_pixmap = None;
     }
 
     if (pSurfaceGLX->pixmap) {
-        XFreePixmap((Display *)ctx->native_dpy, pSurfaceGLX->pixmap);
+        XFreePixmap(ctx->native_dpy, pSurfaceGLX->pixmap);
         pSurfaceGLX->pixmap = None;
     }
 }
@@ -583,12 +621,12 @@ static int bind_pixmap(VADriverContextP ctx, VASurfaceGLXP pSurfaceGLX)
 
     x11_trap_errors();
     pOpenGLVTable->glx_bind_tex_image(
-        (Display *)ctx->native_dpy,
+        ctx->native_dpy,
         pSurfaceGLX->glx_pixmap,
         GLX_FRONT_LEFT_EXT,
         NULL
     );
-    XSync((Display *)ctx->native_dpy, False);
+    XSync(ctx->native_dpy, False);
     if (x11_untrap_errors() != 0) {
         va_glx_error_message("failed to bind pixmap\n");
         return 0;
@@ -608,11 +646,11 @@ static int unbind_pixmap(VADriverContextP ctx, VASurfaceGLXP pSurfaceGLX)
 
     x11_trap_errors();
     pOpenGLVTable->glx_release_tex_image(
-        (Display *)ctx->native_dpy,
+        ctx->native_dpy,
         pSurfaceGLX->glx_pixmap,
         GLX_FRONT_LEFT_EXT
     );
-    XSync((Display *)ctx->native_dpy, False);
+    XSync(ctx->native_dpy, False);
     if (x11_untrap_errors() != 0) {
         va_glx_error_message("failed to release pixmap\n");
         return 0;
@@ -803,7 +841,7 @@ end:
 /* ========================================================================= */
 
 #define INVOKE(ctx, func, args) do {                    \
-        VADriverVTableGLXP vtable = (ctx)->vtable.glx;  \
+        VADriverVTableGLXP vtable = (ctx)->vtable_glx;  \
         if (!vtable->va##func##GLX)                     \
             return VA_STATUS_ERROR_UNIMPLEMENTED;       \
                                                         \
@@ -876,19 +914,25 @@ vaCreateSurfaceGLX_impl_libva(
     gl_get_current_context(&old_cs);
     new_cs = gl_create_context(ctx, &old_cs);
     if (!new_cs)
-        return VA_STATUS_ERROR_ALLOCATION_FAILED;
+        goto error;
     if (!gl_set_current_context(new_cs, NULL))
-        return VA_STATUS_ERROR_OPERATION_FAILED;
+        goto error;
 
     pSurfaceGLX = create_surface(ctx, target, texture);
     if (!pSurfaceGLX)
-        return VA_STATUS_ERROR_ALLOCATION_FAILED;
+        goto error;
 
     pSurfaceGLX->gl_context = new_cs;
     *gl_surface = pSurfaceGLX;
 
     gl_set_current_context(&old_cs, NULL);
     return VA_STATUS_SUCCESS;
+
+error:
+    if (new_cs)
+        gl_destroy_context(new_cs);
+
+    return VA_STATUS_ERROR_ALLOCATION_FAILED;    
 }
 
 static VAStatus
@@ -937,7 +981,7 @@ associate_surface(
         return status;
 
     x11_trap_errors();
-    status = ctx->vtable.vaPutSurface(
+    status = ctx->vtable->vaPutSurface(
         ctx,
         surface,
         (void *)pSurfaceGLX->pixmap,
@@ -946,7 +990,7 @@ associate_surface(
         NULL, 0,
         flags
     );
-    XSync((Display *)ctx->native_dpy, False);
+    XSync(ctx->native_dpy, False);
     if (x11_untrap_errors() != 0)
         return VA_STATUS_ERROR_OPERATION_FAILED;
     if (status != VA_STATUS_SUCCESS)
@@ -962,7 +1006,7 @@ sync_surface(VADriverContextP ctx, VASurfaceGLXP pSurfaceGLX)
     if (pSurfaceGLX->surface == VA_INVALID_SURFACE)
         return VA_STATUS_ERROR_INVALID_SURFACE;
 
-    return ctx->vtable.vaSyncSurface(ctx, pSurfaceGLX->surface);
+    return ctx->vtable->vaSyncSurface(ctx, pSurfaceGLX->surface);
 }
 
 static inline VAStatus
@@ -1058,7 +1102,7 @@ VAStatus va_glx_init_context(VADriverContextP ctx)
     if (glx_ctx->is_initialized)
         return VA_STATUS_SUCCESS;
 
-    if (ctx->vtable.glx && ctx->vtable.glx->vaCopySurfaceGLX) {
+    if (ctx->vtable_glx && ctx->vtable_glx->vaCopySurfaceGLX) {
         vtable->vaCreateSurfaceGLX      = vaCreateSurfaceGLX_impl_driver;
         vtable->vaDestroySurfaceGLX     = vaDestroySurfaceGLX_impl_driver;
         vtable->vaCopySurfaceGLX        = vaCopySurfaceGLX_impl_driver;
@@ -1068,7 +1112,7 @@ VAStatus va_glx_init_context(VADriverContextP ctx)
         vtable->vaDestroySurfaceGLX     = vaDestroySurfaceGLX_impl_libva;
         vtable->vaCopySurfaceGLX        = vaCopySurfaceGLX_impl_libva;
 
-        if (!glXQueryVersion((Display *)ctx->native_dpy, &glx_major, &glx_minor))
+        if (!glXQueryVersion(ctx->native_dpy, &glx_major, &glx_minor))
             return VA_STATUS_ERROR_UNIMPLEMENTED;
 
         if (!check_tfp_extensions(ctx) || !load_tfp_extensions(ctx))
