@@ -26,8 +26,10 @@
 #include "sysdeps.h"
 #include "va.h"
 #include "va_backend.h"
+#include "va_internal.h"
+#include "va_trace.h"
+#include "va_fool.h"
 #include "va_x11.h"
-#include "va_dri.h"
 #include "va_dri2.h"
 #include "va_dricommon.h"
 #include "va_nvctrl.h"
@@ -42,41 +44,31 @@
 #include <fcntl.h>
 #include <errno.h>
 
-static VADisplayContextP pDisplayContexts = NULL;
-
 static int va_DisplayContextIsValid (
     VADisplayContextP pDisplayContext
 )
 {
-    VADisplayContextP ctx = pDisplayContexts;
-
-    while (ctx)
-    {
-	if (ctx == pDisplayContext && pDisplayContext->pDriverContext)
-	    return 1;
-	ctx = ctx->pNext;
-    }
-    return 0;
+    return (pDisplayContext != NULL && 
+            pDisplayContext->pDriverContext != NULL);
 }
 
 static void va_DisplayContextDestroy (
     VADisplayContextP pDisplayContext
 )
 {
-    VADisplayContextP *ctx = &pDisplayContexts;
+    VADriverContextP ctx;
+    struct dri_state *dri_state;
 
-    /* Throw away pDisplayContext */
-    while (*ctx)
-    {
-	if (*ctx == pDisplayContext)
-	{
-	    *ctx = pDisplayContext->pNext;
-	    pDisplayContext->pNext = NULL;
-	    break;
-	}
-	ctx = &((*ctx)->pNext);
-    }
-    free(pDisplayContext->pDriverContext->dri_state);
+    if (pDisplayContext == NULL)
+        return;
+
+    ctx = pDisplayContext->pDriverContext;
+    dri_state = ctx->drm_state;
+
+    if (dri_state && dri_state->close)
+        dri_state->close(ctx);
+
+    free(pDisplayContext->pDriverContext->drm_state);
     free(pDisplayContext->pDriverContext);
     free(pDisplayContext);
 }
@@ -89,20 +81,7 @@ static VAStatus va_DRI2GetDriverName (
 {
     VADriverContextP ctx = pDisplayContext->pDriverContext;
 
-    if (!isDRI2Connected(ctx, driver_name))
-        return VA_STATUS_ERROR_UNKNOWN;
-
-    return VA_STATUS_SUCCESS;
-}
-
-static VAStatus va_DRIGetDriverName (
-    VADisplayContextP pDisplayContext,
-    char **driver_name
-)
-{
-    VADriverContextP ctx = pDisplayContext->pDriverContext;
-
-    if (!isDRI1Connected(ctx, driver_name))
+    if (!va_isDRI2Connected(ctx, driver_name))
         return VA_STATUS_ERROR_UNKNOWN;
 
     return VA_STATUS_SUCCESS;
@@ -117,12 +96,12 @@ static VAStatus va_NVCTRL_GetDriverName (
     int direct_capable, driver_major, driver_minor, driver_patch;
     Bool result;
 
-    result = VA_NVCTRLQueryDirectRenderingCapable((Display *)ctx->native_dpy, ctx->x11_screen,
+    result = VA_NVCTRLQueryDirectRenderingCapable(ctx->native_dpy, ctx->x11_screen,
                                                   &direct_capable);
     if (!result || !direct_capable)
         return VA_STATUS_ERROR_UNKNOWN;
 
-    result = VA_NVCTRLGetClientDriverName((Display *)ctx->native_dpy, ctx->x11_screen,
+    result = VA_NVCTRLGetClientDriverName(ctx->native_dpy, ctx->x11_screen,
                                           &driver_major, &driver_minor,
                                           &driver_patch, driver_name);
     if (!result)
@@ -158,10 +137,10 @@ static VAStatus va_DisplayContextGetDriverName (
 
     if (driver_name)
 	*driver_name = NULL;
-
+    else
+        return VA_STATUS_ERROR_UNKNOWN;
+    
     vaStatus = va_DRI2GetDriverName(pDisplayContext, driver_name);
-    if (vaStatus != VA_STATUS_SUCCESS)
-        vaStatus = va_DRIGetDriverName(pDisplayContext, driver_name);
     if (vaStatus != VA_STATUS_SUCCESS)
         vaStatus = va_NVCTRL_GetDriverName(pDisplayContext, driver_name);
     if (vaStatus != VA_STATUS_SUCCESS)
@@ -174,69 +153,43 @@ VADisplay vaGetDisplay (
     Display *native_dpy /* implementation specific */
 )
 {
-  VADisplay dpy = NULL;
-  VADisplayContextP pDisplayContext = pDisplayContexts;
+    VADisplayContextP pDisplayContext;
+    VADriverContextP  pDriverContext;
+    struct dri_state *dri_state;
 
-  if (!native_dpy)
-      return NULL;
+    if (!native_dpy)
+        return NULL;
 
-  while (pDisplayContext)
-  {
-      if (pDisplayContext->pDriverContext &&
-	  pDisplayContext->pDriverContext->native_dpy == (void *)native_dpy)
-      {
-          dpy = (VADisplay)pDisplayContext;
-          break;
-      }
-      pDisplayContext = pDisplayContext->pNext;
-  }
+    pDisplayContext = va_newDisplayContext();
+    if (!pDisplayContext)
+        return NULL;
 
-  if (!dpy)
-  {
-      /* create new entry */
-      VADriverContextP pDriverContext;
-      struct dri_state *dri_state;
-      pDisplayContext = calloc(1, sizeof(*pDisplayContext));
-      pDriverContext  = calloc(1, sizeof(*pDriverContext));
-      dri_state       = calloc(1, sizeof(*dri_state));
-      if (pDisplayContext && pDriverContext && dri_state)
-      {
-	  pDisplayContext->vadpy_magic = VA_DISPLAY_MAGIC;          
+    pDisplayContext->vaIsValid       = va_DisplayContextIsValid;
+    pDisplayContext->vaDestroy       = va_DisplayContextDestroy;
+    pDisplayContext->vaGetDriverName = va_DisplayContextGetDriverName;
 
-	  pDriverContext->native_dpy       = (void *)native_dpy;
-	  pDisplayContext->pNext           = pDisplayContexts;
-	  pDisplayContext->pDriverContext  = pDriverContext;
-	  pDisplayContext->vaIsValid       = va_DisplayContextIsValid;
-	  pDisplayContext->vaDestroy       = va_DisplayContextDestroy;
-	  pDisplayContext->vaGetDriverName = va_DisplayContextGetDriverName;
-          pDisplayContext->opaque          = NULL;
-	  pDisplayContexts                 = pDisplayContext;
-	  pDriverContext->dri_state 	   = dri_state;
-	  dpy                              = (VADisplay)pDisplayContext;
-      }
-      else
-      {
-	  if (pDisplayContext)
-	      free(pDisplayContext);
-	  if (pDriverContext)
-	      free(pDriverContext);
-          if (dri_state)
-              free(dri_state);
-      }
-  }
-  
-  return dpy;
+    pDriverContext = va_newDriverContext(pDisplayContext);
+    if (!pDriverContext) {
+        free(pDisplayContext);
+        return NULL;
+    }
+
+    pDriverContext->native_dpy   = (void *)native_dpy;
+    pDriverContext->x11_screen   = XDefaultScreen(native_dpy);
+    pDriverContext->display_type = VA_DISPLAY_X11;
+
+    dri_state = calloc(1, sizeof(*dri_state));
+    if (!dri_state) {
+        free(pDisplayContext);
+        free(pDriverContext);
+        return NULL;
+    }
+
+    pDriverContext->drm_state = dri_state;
+
+    return (VADisplay)pDisplayContext;
 }
 
-#define CTX(dpy) (((VADisplayContextP)dpy)->pDriverContext)
-#define CHECK_DISPLAY(dpy) if( !vaDisplayIsValid(dpy) ) { return VA_STATUS_ERROR_INVALID_DISPLAY; }
-
-extern int fool_postp; /* do nothing for vaPutSurface if set */
-extern int trace_flag; /* trace vaPutSurface parameters */
-#define VA_TRACE(trace_func,...)                \
-    if (trace_flag) {                           \
-        trace_func(__VA_ARGS__);                \
-    }
 
 void va_TracePutSurface (
     VADisplay dpy,
@@ -275,17 +228,17 @@ VAStatus vaPutSurface (
 {
   VADriverContextP ctx;
 
-  if (fool_postp)
+  if (va_fool_postp)
       return VA_STATUS_SUCCESS;
 
   CHECK_DISPLAY(dpy);
   ctx = CTX(dpy);
   
-  VA_TRACE(va_TracePutSurface, dpy, surface, (void *)draw, srcx, srcy, srcw, srch,
-           destx, desty, destw, desth,
-           cliprects, number_cliprects, flags );
+  VA_TRACE_LOG(va_TracePutSurface, dpy, surface, (void *)draw, srcx, srcy, srcw, srch,
+               destx, desty, destw, desth,
+               cliprects, number_cliprects, flags );
   
-  return ctx->vtable.vaPutSurface( ctx, surface, (void *)draw, srcx, srcy, srcw, srch,
+  return ctx->vtable->vaPutSurface( ctx, surface, (void *)draw, srcx, srcy, srcw, srch,
                                    destx, desty, destw, desth,
                                    cliprects, number_cliprects, flags );
 }
